@@ -5,14 +5,19 @@
 // SPDX-FileCopyrightText: 2023-2024, Alejandro Colomar <alx@kernel.org>
 // SPDX-License-Identifier: BSD-3-Clause
 
+
 /*
  * is_valid_user_name(), is_valid_group_name() - check the new user/group
  * name for validity;
  * return values:
  *   true  - OK
  *   false - bad name
+ * errors:
+ *   EINVAL	Invalid name characters or sequences
+ *   EOVERFLOW	Name longer than maximum size
  */
 
+
 #include <config.h>
 
 #ident "$Id$"
@@ -20,12 +25,36 @@
 #include <ctype.h>
 #include <errno.h>
 #include <limits.h>
+#include <stdbool.h>
+#include <stddef.h>
+#include <stdint.h>
+#include <sys/param.h>
+#include <unistd.h>
+
 #include "defines.h"
 #include "chkname.h"
+#include "string/strcmp/streq.h"
+
 
 int allow_bad_names = false;
 
-static bool is_valid_name (const char *name)
+
+size_t
+login_name_max_size(void)
+{
+	long  conf;
+
+	errno = 0;
+	conf = sysconf(_SC_LOGIN_NAME_MAX);
+	if (conf == -1 && errno != 0)
+		return LOGIN_NAME_MAX;
+
+	return MIN(conf, PTRDIFF_MAX);
+}
+
+
+static bool
+is_valid_name(const char *name)
 {
 	if (allow_bad_names) {
 		return true;
@@ -49,13 +78,15 @@ static bool is_valid_name (const char *name)
 	      (*name >= 'A' && *name <= 'Z') ||
 	      (*name >= '0' && *name <= '9') ||
 	      *name == '_' ||
-	      *name == '.')) {
+	      *name == '.'))
+	{
+		errno = EINVAL;
 		return false;
 	}
 
 	numeric = isdigit(*name);
 
-	while ('\0' != *++name) {
+	while (!streq(++name, "")) {
 		if (!((*name >= 'a' && *name <= 'z') ||
 		      (*name >= 'A' && *name <= 'Z') ||
 		      (*name >= '0' && *name <= '9') ||
@@ -63,45 +94,46 @@ static bool is_valid_name (const char *name)
 		      *name == '.' ||
 		      *name == '-' ||
 		      (*name == '$' && name[1] == '\0')
-		     )) {
+		     ))
+		{
+			errno = EINVAL;
 			return false;
 		}
 		numeric &= isdigit(*name);
 	}
 
-	return !numeric;
+	if (numeric) {
+		errno = EINVAL;
+		return false;
+	}
+
+	return true;
 }
 
 
 bool
 is_valid_user_name(const char *name)
 {
-	long    conf;
-	size_t  maxsize;
-
-	errno = 0;
-	conf = sysconf(_SC_LOGIN_NAME_MAX);
-
-	if (conf == -1 && errno != 0)
-		maxsize = LOGIN_NAME_MAX;
-	else
-		maxsize = conf;
-
-	if (strlen(name) >= maxsize)
+	if (strlen(name) >= login_name_max_size()) {
+		errno = EOVERFLOW;
 		return false;
+	}
 
 	return is_valid_name(name);
 }
 
 
-bool is_valid_group_name (const char *name)
+bool
+is_valid_group_name(const char *name)
 {
 	/*
 	 * Arbitrary limit for group names.
 	 * HP-UX 10 limits to 16 characters
 	 */
 	if (   (GROUP_NAME_MAX_LENGTH > 0)
-	    && (strlen (name) > GROUP_NAME_MAX_LENGTH)) {
+	    && (strlen (name) > GROUP_NAME_MAX_LENGTH))
+	{
+		errno = EOVERFLOW;
 		return false;
 	}
 
